@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- * Copyright (C) 2002 Mikael Hallendal <micke@imendio.com>
+ * Copyright (C) 2009  Shaun McCance  <shaunm@gnome.org
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,7 +17,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * Author: Mikael Hallendal <micke@imendio.com>
+ * Author: Shaun McCance <shaunm@gnome.org>
  */
 
 #include <config.h>
@@ -29,11 +29,17 @@
 
 #include "yelp-uri.h"
 
+GMainLoop *loop;
+
 static void 
-print_uri (YelpUri *uri, GOutputStream *stream)
+print_uri (gchar *orig, YelpUri *uri, GOutputStream *stream)
 {
+    GFile *file;
     gchar *type, *tmp, **tmpv, *out;
 
+    g_output_stream_write (stream, orig, strlen (orig), NULL, NULL);
+    g_output_stream_write (stream, "\n", 1, NULL, NULL);
+
     switch (yelp_uri_get_document_type (uri)) {
     case YELP_URI_DOCUMENT_TYPE_DOCBOOK:
         type = "DOCBOOK";
@@ -71,31 +77,49 @@ print_uri (YelpUri *uri, GOutputStream *stream)
     case YELP_URI_DOCUMENT_TYPE_ERROR:
         type = "ERROR";
         break;
-    case YELP_URI_DOCUMENT_TYPE_UNKNOWN:
-        type = "UNKNOWN";
+    case YELP_URI_DOCUMENT_TYPE_UNRESOLVED:
+        type = "UNRESOLVED";
         break;
     }
 
-    out = g_strdup_printf ("TYPE:  %s\n", type);
+    out = g_strdup_printf ("DOCUMENT TYPE: %s\n", type);
     g_output_stream_write (stream, out, strlen (out), NULL, NULL);
     g_free (out);
 
-    tmp = yelp_uri_get_base_uri (uri);
+    tmp = yelp_uri_get_document_uri (uri);
     if (tmp) {
-        out = g_strdup_printf ("URI:   %s\n", tmp);
+        out = g_strdup_printf ("DOCUMENT URI:  %s\n", tmp);
         g_output_stream_write (stream, out, strlen (out), NULL, NULL);
         g_free (out);
         g_free (tmp);
     }
 
+    tmp = yelp_uri_get_canonical_uri (uri);
+    if (tmp) {
+        out = g_strdup_printf ("CANONICAL URI: %s\n", tmp);
+        g_output_stream_write (stream, out, strlen (out), NULL, NULL);
+        g_free (out);
+        g_free (tmp);
+    }
+
+    file = yelp_uri_get_file (uri);
+    if (file) {
+        tmp = g_file_get_uri (file);
+        out = g_strdup_printf ("FILE URI:      %s\n", tmp);
+        g_output_stream_write (stream, out, strlen (out), NULL, NULL);
+        g_free (out);
+        g_free (tmp);
+        g_object_unref (file);
+    }
+
     tmpv = yelp_uri_get_search_path (uri);
     if (tmpv) {
         int i;
         for (i = 0; tmpv[i]; i++) {
             if (i == 0)
-                out = g_strdup_printf ("PATH:  %s\n", tmpv[i]);
+                out = g_strdup_printf ("SEARCH PATH:   %s\n", tmpv[i]);
             else
-                out = g_strdup_printf ("       %s\n", tmpv[i]);
+                out = g_strdup_printf ("               %s\n", tmpv[i]);
             g_output_stream_write (stream, out, strlen (out), NULL, NULL);
             g_free (out);
         }
@@ -104,7 +128,7 @@ print_uri (YelpUri *uri, GOutputStream *stream)
 
     tmp = yelp_uri_get_page_id (uri);
     if (tmp) {
-        out = g_strdup_printf ("PAGE:  %s\n", tmp);
+        out = g_strdup_printf ("PAGE ID:       %s\n", tmp);
         g_output_stream_write (stream, out, strlen (out), NULL, NULL);
         g_free (out);
         g_free (tmp);
@@ -112,11 +136,13 @@ print_uri (YelpUri *uri, GOutputStream *stream)
 
     tmp = yelp_uri_get_frag_id (uri);
     if (tmp) {
-        out = g_strdup_printf ("FRAG:  %s\n", tmp);
+        out = g_strdup_printf ("FRAG ID:       %s\n", tmp);
         g_output_stream_write (stream, out, strlen (out), NULL, NULL);
         g_free (out);
         g_free (tmp);
     }
+
+    g_output_stream_write (stream, "\0", 1, NULL, NULL);
 }
 
 static void run_test (gconstpointer data)
@@ -125,10 +151,11 @@ static void run_test (gconstpointer data)
     gchar contents[1024];
     gsize bytes;
     gchar *curi, *newline;
+    gchar **uriv;
     GFile *file = G_FILE (data);
     YelpUri *uri;
     GOutputStream *outstream;
-    gchar *test, *out;
+    gchar *out;
 
     stream = g_file_read (file, NULL, NULL);
     g_assert (g_input_stream_read_all (G_INPUT_STREAM (stream),
@@ -136,12 +163,23 @@ static void run_test (gconstpointer data)
                                        NULL, NULL));
     newline = strchr (contents, '\n');
     curi = g_strndup (contents, newline - contents);
-    uri = yelp_uri_resolve (curi);
+    uriv = g_strsplit (curi, " ", 2);
+    uri = yelp_uri_new (uriv[0]);
+    if (uriv[1] != NULL)
+        uri = yelp_uri_new_relative (uri, uriv[1]);
+    g_strfreev (uriv);
+
+    yelp_uri_resolve (uri);
+
+    while (!yelp_uri_is_resolved (uri))
+        while (g_main_context_pending (NULL))
+             g_main_context_iteration (NULL, FALSE);
+
     outstream = g_memory_output_stream_new (NULL, 0, g_realloc, g_free);
-    print_uri (uri, outstream);
+    print_uri (curi, uri, outstream);
     out = (gchar *) g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (outstream));
-    test = g_strndup (newline + 1, bytes - (newline - contents) - 1);
-    g_assert_cmpstr (out, ==, test);
+    g_free (curi);
+    g_assert (!strncmp (out, contents, bytes));
 }
 
 static int
@@ -175,6 +213,15 @@ run_all_tests (int argc, char **argv)
     return g_test_run ();
 }
 
+static void
+uri_resolved (YelpUri *uri, gchar *orig)
+{
+    GOutputStream *stream = g_unix_output_stream_new (1, FALSE);
+    print_uri (orig, uri, stream);
+    g_object_unref (uri);
+    g_main_loop_quit (loop);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -182,28 +229,37 @@ main (int argc, char **argv)
     YelpUri *uri = NULL;
 
     g_type_init ();
+    g_thread_init (NULL);
     g_log_set_always_fatal (G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL);
         
-    g_test_init (&argc, &argv);
     if (argc < 2) {
+        g_test_init (&argc, &argv, NULL);
         return run_all_tests (argc, argv);
     }
     else {
         if (argc > 2) {
-            parent = yelp_uri_resolve (argv[1]);
-            uri = yelp_uri_resolve_relative (parent, argv[2]);
+            parent = yelp_uri_new (argv[1]);
+            uri = yelp_uri_new_relative (parent, argv[2]);
         } else {
-            uri = yelp_uri_resolve (argv[1]);
+            uri = yelp_uri_new (argv[1]);
         }
         if (uri) {
-            GOutputStream *stream = g_unix_output_stream_new (1, FALSE);
-            print_uri (uri, stream);
-            g_object_unref (uri);
+            gchar *orig;
+            /* We leak orig if argc > 2.  I don't care. */
+            if (argc > 2)
+                orig = g_strconcat (argv[1], " ", argv[2], NULL);
+            else
+                orig = argv[1];
+            g_signal_connect (uri, "resolved", G_CALLBACK (uri_resolved), orig);
+            yelp_uri_resolve (uri);
         }
         if (parent) {
             g_object_unref (parent);
         }
     }
 
+    loop = g_main_loop_new (NULL, FALSE);
+    g_main_loop_run (loop);
+
     return 0;
 }
